@@ -1000,12 +1000,19 @@ contact_list_editor_combo_box_changed_cb (GtkWidget *widget)
 	client = E_CLIENT (editor->priv->book_client);
 	client_source = e_client_get_source (client);
 
-	if (!e_source_equal (client_source, active_source))
+	if (!e_source_equal (client_source, active_source)) {
+		ConnectClosure *connect_closure;
+
+		connect_closure = g_slice_new0 (ConnectClosure);
+		connect_closure->editor = g_object_ref (editor);
+		connect_closure->source = g_object_ref (active_source);
+
 		e_client_combo_box_get_client (
 			E_CLIENT_COMBO_BOX (widget),
 			active_source, NULL,
 			contact_list_editor_get_client_cb,
-			g_object_ref (editor));
+			connect_closure);
+	}
 
 	g_object_unref (active_source);
 }
@@ -1584,7 +1591,7 @@ contact_list_editor_contact_added (EABEditor *editor,
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		return;
 
-	eab_error_dialog (NULL, _("Error adding list"), error);
+	eab_error_dialog (NULL, eab_editor_get_window (editor), _("Error adding list"), error);
 }
 
 static void
@@ -1599,7 +1606,7 @@ contact_list_editor_contact_modified (EABEditor *editor,
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		return;
 
-	eab_error_dialog (NULL, _("Error modifying list"), error);
+	eab_error_dialog (NULL, eab_editor_get_window (editor), _("Error modifying list"), error);
 }
 
 static void
@@ -1614,7 +1621,7 @@ contact_list_editor_contact_deleted (EABEditor *editor,
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		return;
 
-	eab_error_dialog (NULL, _("Error removing list"), error);
+	eab_error_dialog (NULL, eab_editor_get_window (editor), _("Error removing list"), error);
 }
 
 static void
