@@ -36,7 +36,9 @@
 	((obj), E_TYPE_ADDRESSBOOK_MODEL, EAddressbookModelPrivate))
 
 struct _EAddressbookModelPrivate {
-	ESourceRegistry *registry;
+	EClientCache *client_cache;
+	gulong client_notify_readonly_handler_id;
+
 	EBookClient *book_client;
 	gchar *query_str;
 	EBookClientView *client_view;
@@ -50,7 +52,6 @@ struct _EAddressbookModelPrivate {
 	gulong remove_contact_id;
 	gulong modify_contact_id;
 	gulong status_message_id;
-	gulong writable_status_id;
 	gulong view_complete_id;
 	guint remove_status_id;
 
@@ -62,9 +63,9 @@ struct _EAddressbookModelPrivate {
 enum {
 	PROP_0,
 	PROP_CLIENT,
+	PROP_CLIENT_CACHE,
 	PROP_EDITABLE,
-	PROP_QUERY,
-	PROP_REGISTRY
+	PROP_QUERY
 };
 
 enum {
@@ -335,14 +336,18 @@ view_complete_cb (EBookClientView *client_view,
 }
 
 static void
-readonly_cb (EBookClient *book_client,
-             GParamSpec *pspec,
-             EAddressbookModel *model)
+addressbook_model_client_notify_readonly_cb (EClientCache *client_cache,
+                                             EClient *client,
+                                             GParamSpec *pspec,
+                                             EAddressbookModel *model)
 {
-	gboolean editable;
+	if (!E_IS_BOOK_CLIENT (client))
+		return;
 
-	editable = !e_client_is_readonly (E_CLIENT (book_client));
-	e_addressbook_model_set_editable (model, editable);
+	if (E_BOOK_CLIENT (client) == model->priv->book_client) {
+		gboolean editable = !e_client_is_readonly (client);
+		e_addressbook_model_set_editable (model, editable);
+	}
 }
 
 static void
@@ -359,7 +364,7 @@ client_view_ready_cb (GObject *source_object,
 		client_view = NULL;
 
 	if (error) {
-		eab_error_dialog (NULL, _("Error getting book view"), error);
+		eab_error_dialog (NULL, NULL, _("Error getting book view"), error);
 		g_error_free (error);
 		return;
 	}
@@ -454,13 +459,13 @@ remove_status_cb (gpointer data)
 }
 
 static void
-addressbook_model_set_registry (EAddressbookModel *model,
-                                ESourceRegistry *registry)
+addressbook_model_set_client_cache (EAddressbookModel *model,
+                                    EClientCache *client_cache)
 {
-	g_return_if_fail (E_IS_SOURCE_REGISTRY (registry));
-	g_return_if_fail (model->priv->registry == NULL);
+	g_return_if_fail (E_IS_CLIENT_CACHE (client_cache));
+	g_return_if_fail (model->priv->client_cache == NULL);
 
-	model->priv->registry = g_object_ref (registry);
+	model->priv->client_cache = g_object_ref (client_cache);
 }
 
 static void
@@ -476,6 +481,12 @@ addressbook_model_set_property (GObject *object,
 				g_value_get_object (value));
 			return;
 
+		case PROP_CLIENT_CACHE:
+			addressbook_model_set_client_cache (
+				E_ADDRESSBOOK_MODEL (object),
+				g_value_get_object (value));
+			return;
+
 		case PROP_EDITABLE:
 			e_addressbook_model_set_editable (
 				E_ADDRESSBOOK_MODEL (object),
@@ -487,12 +498,6 @@ addressbook_model_set_property (GObject *object,
 				E_ADDRESSBOOK_MODEL (object),
 				g_value_get_string (value));
 			return;
-
-		case PROP_REGISTRY:
-			addressbook_model_set_registry (
-				E_ADDRESSBOOK_MODEL (object),
-				g_value_get_object (value));
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -512,6 +517,12 @@ addressbook_model_get_property (GObject *object,
 				E_ADDRESSBOOK_MODEL (object)));
 			return;
 
+		case PROP_CLIENT_CACHE:
+			g_value_set_object (
+				value, e_addressbook_model_get_client_cache (
+				E_ADDRESSBOOK_MODEL (object)));
+			return;
+
 		case PROP_EDITABLE:
 			g_value_set_boolean (
 				value, e_addressbook_model_get_editable (
@@ -523,12 +534,6 @@ addressbook_model_get_property (GObject *object,
 				value, e_addressbook_model_get_query (
 				E_ADDRESSBOOK_MODEL (object)));
 			return;
-
-		case PROP_REGISTRY:
-			g_value_set_object (
-				value, e_addressbook_model_get_registry (
-				E_ADDRESSBOOK_MODEL (object)));
-			return;
 	}
 
 	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -542,17 +547,16 @@ addressbook_model_dispose (GObject *object)
 	remove_book_view (model);
 	free_data (model);
 
-	if (model->priv->book_client) {
-		if (model->priv->writable_status_id)
-			g_signal_handler_disconnect (
-				model->priv->book_client,
-				model->priv->writable_status_id);
-		model->priv->writable_status_id = 0;
-
-		g_object_unref (model->priv->book_client);
-		model->priv->book_client = NULL;
+	if (model->priv->client_notify_readonly_handler_id > 0) {
+		g_signal_handler_disconnect (
+			model->priv->client_cache,
+			model->priv->client_notify_readonly_handler_id);
+		model->priv->client_notify_readonly_handler_id = 0;
 	}
 
+	g_clear_object (&model->priv->client_cache);
+	g_clear_object (&model->priv->book_client);
+
 	if (model->priv->query_str) {
 		g_free (model->priv->query_str);
 		model->priv->query_str = NULL;
@@ -575,6 +579,27 @@ addressbook_model_finalize (GObject *object)
 	G_OBJECT_CLASS (e_addressbook_model_parent_class)->finalize (object);
 }
 
+static void
+addressbook_model_constructed (GObject *object)
+{
+	EAddressbookModel *model;
+	EClientCache *client_cache;
+	gulong handler_id;
+
+	model = E_ADDRESSBOOK_MODEL (object);
+
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (e_addressbook_model_parent_class)->constructed (object);
+
+	client_cache = e_addressbook_model_get_client_cache (model);
+
+	handler_id = g_signal_connect (
+		client_cache, "client-notify::readonly",
+		G_CALLBACK (addressbook_model_client_notify_readonly_cb),
+		model);
+	model->priv->client_notify_readonly_handler_id = handler_id;
+}
+
 static void
 e_addressbook_model_class_init (EAddressbookModelClass *class)
 {
@@ -587,6 +612,7 @@ e_addressbook_model_class_init (EAddressbookModelClass *class)
 	object_class->get_property = addressbook_model_get_property;
 	object_class->dispose = addressbook_model_dispose;
 	object_class->finalize = addressbook_model_finalize;
+	object_class->constructed = addressbook_model_constructed;
 
 	g_object_class_install_property (
 		object_class,
@@ -599,6 +625,18 @@ e_addressbook_model_class_init (EAddressbookModelClass *class)
 			G_PARAM_READWRITE |
 			G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property (
+		object_class,
+		PROP_CLIENT_CACHE,
+		g_param_spec_object (
+			"client-cache",
+			"Client Cache",
+			"Shared EClient instances",
+			E_TYPE_CLIENT_CACHE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT_ONLY |
+			G_PARAM_STATIC_STRINGS));
+
 	g_object_class_install_property (
 		object_class,
 		PROP_EDITABLE,
@@ -622,18 +660,6 @@ e_addressbook_model_class_init (EAddressbookModelClass *class)
 			G_PARAM_CONSTRUCT |
 			G_PARAM_STATIC_STRINGS));
 
-	g_object_class_install_property (
-		object_class,
-		PROP_REGISTRY,
-		g_param_spec_object (
-			"registry",
-			"Registry",
-			"Data source registry",
-			E_TYPE_SOURCE_REGISTRY,
-			G_PARAM_READWRITE |
-			G_PARAM_CONSTRUCT_ONLY |
-			G_PARAM_STATIC_STRINGS));
-
 	signals[WRITABLE_STATUS] = g_signal_new (
 		"writable_status",
 		G_OBJECT_CLASS_TYPE (object_class),
@@ -743,21 +769,21 @@ e_addressbook_model_init (EAddressbookModel *model)
 }
 
 EAddressbookModel *
-e_addressbook_model_new (ESourceRegistry *registry)
+e_addressbook_model_new (EClientCache *client_cache)
 {
-	g_return_val_if_fail (E_IS_SOURCE_REGISTRY (registry), NULL);
+	g_return_val_if_fail (E_IS_CLIENT_CACHE (client_cache), NULL);
 
 	return g_object_new (
 		E_TYPE_ADDRESSBOOK_MODEL,
-		"registry", registry, NULL);
+		"client-cache", client_cache, NULL);
 }
 
-ESourceRegistry *
-e_addressbook_model_get_registry (EAddressbookModel *model)
+EClientCache *
+e_addressbook_model_get_client_cache (EAddressbookModel *model)
 {
 	g_return_val_if_fail (E_IS_ADDRESSBOOK_MODEL (model), NULL);
 
-	return model->priv->registry;
+	return model->priv->client_cache;
 }
 
 EContact *
@@ -873,26 +899,12 @@ e_addressbook_model_set_client (EAddressbookModel *model,
 	if (model->priv->book_client == book_client)
 		return;
 
-	if (model->priv->book_client != NULL) {
-		if (model->priv->book_client == book_client)
-			return;
-
-		if (model->priv->writable_status_id != 0)
-			g_signal_handler_disconnect (
-				model->priv->book_client,
-				model->priv->writable_status_id);
-		model->priv->writable_status_id = 0;
-
+	if (model->priv->book_client != NULL)
 		g_object_unref (model->priv->book_client);
-	}
 
 	model->priv->book_client = g_object_ref (book_client);
 	model->priv->first_get_view = TRUE;
 
-	model->priv->writable_status_id = g_signal_connect (
-		book_client, "notify::readonly",
-		G_CALLBACK (readonly_cb), model);
-
 	editable = !e_client_is_readonly (E_CLIENT (book_client));
 	e_addressbook_model_set_editable (model, editable);
 
