@@ -77,6 +77,7 @@ typedef enum {
 
 void
 eab_error_dialog (EAlertSink *alert_sink,
+		  GtkWindow *parent,
                   const gchar *msg,
                   const GError *error)
 {
@@ -86,11 +87,15 @@ eab_error_dialog (EAlertSink *alert_sink,
 				alert_sink,
 				"addressbook:generic-error",
 				msg, error->message, NULL);
-		else
+		else {
+			if (!parent)
+				parent = e_shell_get_active_window (NULL);
+
 			e_alert_run_dialog_for_args (
-				e_shell_get_active_window (NULL),
+				parent,
 				"addressbook:generic-error",
 				msg, error->message, NULL);
+		}
 	}
 }
 
@@ -469,7 +474,9 @@ process_unref (ContactCopyProcess *process)
 			if (process->count > 0)
 				return;
 		}
-		e_client_util_free_object_slist (process->contacts);
+		g_slist_free_full (
+			process->contacts,
+			(GDestroyNotify) g_object_unref);
 		g_object_unref (process->source);
 		g_object_unref (process->destination);
 		g_object_unref (process->registry);
@@ -488,7 +495,7 @@ contact_added_cb (EBookClient *book_client,
 	if (error && !g_error_matches (error, E_CLIENT_ERROR, E_CLIENT_ERROR_CANCELLED) &&
 	    !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 		process->book_status = FALSE;
-		eab_error_dialog (process->alert_sink, _("Error adding contact"), error);
+		eab_error_dialog (process->alert_sink, NULL, _("Error adding contact"), error);
 	} else if (g_error_matches (error, E_CLIENT_ERROR, E_CLIENT_ERROR_CANCELLED) ||
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 		process->book_status = FALSE;
@@ -522,28 +529,27 @@ do_copy (gpointer data,
 }
 
 static void
-book_loaded_cb (GObject *source_object,
-                GAsyncResult *result,
-                gpointer user_data)
+book_client_connect_cb (GObject *source_object,
+                        GAsyncResult *result,
+                        gpointer user_data)
 {
-	ESource *destination = E_SOURCE (source_object);
 	ContactCopyProcess *process = user_data;
-	EClient *client = NULL;
+	EClient *client;
 	GError *error = NULL;
 
-	e_client_utils_open_new_finish (destination, result, &client, &error);
+	client = e_book_client_connect_finish (result, &error);
+
+	/* Sanity check. */
+	g_return_if_fail (
+		((client != NULL) && (error == NULL)) ||
+		((client == NULL) && (error != NULL)));
 
 	if (error != NULL) {
-		g_warn_if_fail (client == NULL);
-		g_warning (
-			"%s: Failed to open destination client: %s",
-			G_STRFUNC, error->message);
+		g_warning ("%s: %s", G_STRFUNC, error->message);
 		g_error_free (error);
 		goto exit;
 	}
 
-	g_return_if_fail (E_IS_CLIENT (client));
-
 	process->destination = E_BOOK_CLIENT (client);
 	process->book_status = TRUE;
 	g_slist_foreach (process->contacts, do_copy, process);
@@ -610,64 +616,8 @@ eab_transfer_contacts (ESourceRegistry *registry,
 	process->alert_sink = alert_sink;
 	process->delete_from_source = delete_from_source;
 
-	e_client_utils_open_new (
-		destination, E_CLIENT_SOURCE_TYPE_CONTACTS, FALSE, NULL,
-		book_loaded_cb, process);
-}
-
-/* To parse something like...
- * =?UTF-8?Q?=E0=A4=95=E0=A4=95=E0=A4=AC=E0=A5=82=E0=A5=8B=E0=A5=87?=\t\n=?UTF-8?Q?=E0=A4=B0?=\t\n<aa@aa.ccom>
- * and return the decoded representation of name & email parts. */
-gboolean
-eab_parse_qp_email (const gchar *string,
-                    gchar **name,
-                    gchar **email)
-{
-	struct _camel_header_address *address;
-	gboolean res = FALSE;
-
-	address = camel_header_address_decode (string, "UTF-8");
-
-	if (!address)
-		return FALSE;
-
-	/* report success only when we have filled both name and email address */
-	if (address->type == CAMEL_HEADER_ADDRESS_NAME  && address->name && *address->name && address->v.addr && *address->v.addr) {
-		*name = g_strdup (address->name);
-		*email = g_strdup (address->v.addr);
-		res = TRUE;
-	}
-
-	camel_header_address_unref (address);
-
-	return res;
-}
-
-/* This is only wrapper to parse_qp_mail, it decodes string and if returned TRUE,
- * then makes one string and returns it, otherwise returns NULL.
- * Returned string is usable to place directly into GtkHtml stream.
- * Returned value should be freed with g_free. */
-gchar *
-eab_parse_qp_email_to_html (const gchar *string)
-{
-	gchar *name = NULL, *mail = NULL;
-	gchar *html_name, *html_mail;
-	gchar *value;
-
-	if (!eab_parse_qp_email (string, &name, &mail))
-		return NULL;
-
-	html_name = e_text_to_html (name, 0);
-	html_mail = e_text_to_html (mail, E_TEXT_TO_HTML_CONVERT_ADDRESSES);
-
-	value = g_strdup_printf ("%s &lt;%s&gt;", html_name, html_mail);
-
-	g_free (html_name);
-	g_free (html_mail);
-	g_free (name);
-	g_free (mail);
-
-	return value;
+	e_book_client_connect (
+		destination, NULL, book_client_connect_cb, process);
 }
 
 /*
