@@ -684,6 +684,9 @@ remove_alarms (CompQueuedAlarms *cqa,
 	GSList *l;
 
 	debug (("Removing for %p", cqa));
+
+	tray_list_remove_cqa (cqa);
+
 	for (l = cqa->queued_alarms; l;) {
 		QueuedAlarm *qa;
 
@@ -1077,6 +1080,7 @@ typedef struct {
 	ECalClientView *view;
 	GdkPixbuf *image;
 	GtkTreeIter iter;
+	gboolean is_in_tree;
 } TrayIconData;
 
 static void
@@ -1194,10 +1198,11 @@ tray_list_remove_cqa_async (struct _tray_cqa_msg *msg)
 		if (tray_data->cqa == cqa) {
 			debug (("Found"));
 			tray_icons_list = g_list_delete_link (tray_icons_list, tmp);
-			if (alarm_notifications_dialog) {
+			if (alarm_notifications_dialog && tray_data->is_in_tree) {
 				model = gtk_tree_view_get_model (
 					GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
 				gtk_list_store_remove (GTK_LIST_STORE (model), &(tray_data->iter));
+				tray_data->is_in_tree = FALSE;
 			}
 			free_tray_icon_data (tray_data);
 		}
@@ -1217,10 +1222,11 @@ tray_list_remove_cqa_async (struct _tray_cqa_msg *msg)
 
 			model = gtk_tree_view_get_model (
 				GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
-			gtk_tree_model_get_iter_first (model, &iter);
-			sel = gtk_tree_view_get_selection (
-				GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
-			gtk_tree_selection_select_iter (sel, &iter);
+			if (gtk_tree_model_get_iter_first (model, &iter)) {
+				sel = gtk_tree_view_get_selection (
+					GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
+				gtk_tree_selection_select_iter (sel, &iter);
+			}
 		}
 	}
 
@@ -1255,6 +1261,11 @@ tray_list_remove_async (Message *msg)
 			gboolean status;
 
 			tray_icons_list = g_list_remove_link (tray_icons_list, list);
+			if (alarm_notifications_dialog && tray_data->is_in_tree) {
+				GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
+				gtk_list_store_remove (GTK_LIST_STORE (model), &tray_data->iter);
+				tray_data->is_in_tree = FALSE;
+			}
 			status = remove_queued_alarm (
 				tray_data->cqa,
 				tray_data->alarm_id, FALSE, TRUE);
@@ -1350,6 +1361,7 @@ notify_dialog_cb (AlarmNotifyResult result,
 			/* We can also use tray_data->iter */
 			if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
 				gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+				tray_data->is_in_tree = FALSE;
 				if (!gtk_tree_model_get_iter_first (model, &iter)) {
 					/* We removed the last one */
 					gtk_widget_destroy (alarm_notifications_dialog->dialog);
@@ -1376,12 +1388,13 @@ notify_dialog_cb (AlarmNotifyResult result,
 		break;
 
 	case ALARM_NOTIFY_DISMISS:
-		if (alarm_notifications_dialog) {
+		if (alarm_notifications_dialog && tray_data->is_in_tree) {
 			GtkTreeModel *model;
 
 			model = gtk_tree_view_get_model (
 				GTK_TREE_VIEW (alarm_notifications_dialog->treeview));
 			gtk_list_store_remove (GTK_LIST_STORE (model), &tray_data->iter);
+			tray_data->is_in_tree = FALSE;
 		}
 		break;
 
@@ -1462,6 +1475,8 @@ open_alarm_dialog (TrayIconData *tray_data)
 				tray_data->location,
 				notify_dialog_cb, tray_data);
 
+			tray_data->is_in_tree = TRUE;
+
 			gtk_tree_selection_select_iter (
 				selection, &tray_data->iter);
 
@@ -1578,6 +1593,39 @@ tray_list_add_new (TrayIconData *data)
 	message_push ((Message *) msg);
 }
 
+static void
+alarm_queue_get_alarm_summary (ECalClient *cal_client,
+			       ECalComponent *comp,
+                               const ECalComponentAlarmInstance *instance,
+                               ECalComponentText *text,
+                               ECalComponentAlarm **palarm)
+{
+	g_return_if_fail (comp != NULL);
+	g_return_if_fail (instance != NULL);
+	g_return_if_fail (instance->auid != NULL);
+	g_return_if_fail (text != NULL);
+	g_return_if_fail (palarm != NULL);
+
+	text->value = NULL;
+
+	if (e_client_check_capability (E_CLIENT (cal_client), "alarm-description")) {
+		*palarm = e_cal_component_get_alarm (comp, instance->auid);
+		if (*palarm) {
+			e_cal_component_alarm_get_description (*palarm, text);
+			if (!text->value || !*text->value) {
+				text->value = NULL;
+				e_cal_component_alarm_free (*palarm);
+				*palarm = NULL;
+			}
+		} else {
+			*palarm = NULL;
+		}
+	}
+
+	if (!text->value)
+		e_cal_component_get_summary (comp, text);
+}
+
 /* Performs notification of a display alarm */
 static void
 display_notification (time_t trigger,
@@ -1587,6 +1635,7 @@ display_notification (time_t trigger,
 {
 	QueuedAlarm *qa;
 	ECalComponent *comp;
+	ECalComponentAlarm *comp_alarm = NULL;
 	const gchar *summary, *description, *location;
 	TrayIconData *tray_data;
 	ECalComponentText text;
@@ -1603,7 +1652,7 @@ display_notification (time_t trigger,
 		return;
 
 	/* get a sensible description for the event */
-	e_cal_component_get_summary (comp, &text);
+	alarm_queue_get_alarm_summary (cqa->parent_client->cal_client, comp, qa->instance, &text, &comp_alarm);
 	e_cal_component_get_organizer (comp, &organiser);
 
 	if (text.value)
@@ -1662,11 +1711,12 @@ display_notification (time_t trigger,
 	tray_data->trigger = trigger;
 	tray_data->cqa = cqa;
 	tray_data->alarm_id = alarm_id;
-	tray_data->comp = g_object_ref (e_cal_component_clone (comp));
+	tray_data->comp = e_cal_component_clone (comp);
 	tray_data->cal_client = cqa->parent_client->cal_client;
 	tray_data->view = g_object_ref (cqa->parent_client->view);
 	tray_data->blink_state = FALSE;
 	tray_data->snooze_set = FALSE;
+	tray_data->is_in_tree = FALSE;
 	g_object_ref (tray_data->cal_client);
 
 	/* Task to add tray_data to the global tray_icon_list */
@@ -1685,6 +1735,8 @@ display_notification (time_t trigger,
 		gtk_status_icon_set_tooltip_text (tray_icon, str);
 	}
 
+	if (comp_alarm)
+		e_cal_component_alarm_free (comp_alarm);
 	g_free (start_str);
 	g_free (end_str);
 	g_free (alarm_str);
@@ -1719,6 +1771,7 @@ popup_notification (time_t trigger,
 {
 	QueuedAlarm *qa;
 	ECalComponent *comp;
+	ECalComponentAlarm *comp_alarm = NULL;
 	const gchar *summary, *location;
 	ECalComponentText text;
 	gchar *str, *start_str, *end_str, *alarm_str, *time_str;
@@ -1737,7 +1790,7 @@ popup_notification (time_t trigger,
 		notify_init (_("Evolution Reminders"));
 
 	/* get a sensible description for the event */
-	e_cal_component_get_summary (comp, &text);
+	alarm_queue_get_alarm_summary (cqa->parent_client->cal_client, comp, qa->instance, &text, &comp_alarm);
 	e_cal_component_get_organizer (comp, &organiser);
 
 	if (text.value)
@@ -1792,6 +1845,8 @@ popup_notification (time_t trigger,
 	if (!notify_notification_show (notify, NULL))
 		g_warning ("Could not send notification to daemon\n");
 
+	if (comp_alarm)
+		e_cal_component_alarm_free (comp_alarm);
 	/* create the private structure */
 	g_free (start_str);
 	g_free (end_str);
@@ -1867,35 +1922,12 @@ mail_notification (time_t trigger,
                    CompQueuedAlarms *cqa,
                    gpointer alarm_id)
 {
-	GtkWidget *container;
-	GtkWidget *dialog;
-	GtkWidget *label;
-
-	/* FIXME */
-
-	debug (("..."));
-
 	if (!e_client_check_capability (
 		E_CLIENT (cqa->parent_client->cal_client),
 		CAL_STATIC_CAPABILITY_NO_EMAIL_ALARMS))
 		return;
 
-	dialog = gtk_dialog_new_with_buttons (
-		_("Warning"), NULL, 0,
-		GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
-		NULL);
-	label = gtk_label_new (
-		_("Evolution does not support calendar reminders with\n"
-		"email notifications yet, but this reminder was\n"
-		"configured to send an email.  Evolution will display\n"
-		"a normal reminder dialog box instead."));
-	gtk_widget_show (label);
-
-	container = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-	gtk_box_pack_start (GTK_BOX (container), label, TRUE, TRUE, 4);
-
-	gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	/* FIXME Implement this. */
 }
 
 /* Performs notification of a procedure alarm */
