@@ -178,11 +178,9 @@ window_unmap_cb (GtkWindow *window,
 	if (data->timeout_id > 0)
 		g_source_remove (data->timeout_id);
 
-	/* It's too late to record the window position.
-	 * gtk_window_get_position() will report (0, 0). */
-	data->flags &= ~E_RESTORE_WINDOW_POSITION;
-
-	window_update_settings (data);
+	/* Reset the flags so the window position and size are not
+	 * accidentally reverted to their default value at the next run. */
+	data->flags = 0;
 
 	return FALSE;
 }
@@ -1497,6 +1495,256 @@ e_get_weekday_name (GDateWeekday weekday,
 	return abbreviated ? abbr_names[weekday] : full_names[weekday];
 }
 
+/**
+ * e_weekday_get_next:
+ * @weekday: a #GDateWeekday
+ *
+ * Returns the #GDateWeekday after @weekday.
+ *
+ * Returns: the day after @weekday
+ **/
+GDateWeekday
+e_weekday_get_next (GDateWeekday weekday)
+{
+	GDateWeekday next;
+
+	/* Verbose for readability. */
+	switch (weekday) {
+		case G_DATE_MONDAY:
+			next = G_DATE_TUESDAY;
+			break;
+		case G_DATE_TUESDAY:
+			next = G_DATE_WEDNESDAY;
+			break;
+		case G_DATE_WEDNESDAY:
+			next = G_DATE_THURSDAY;
+			break;
+		case G_DATE_THURSDAY:
+			next = G_DATE_FRIDAY;
+			break;
+		case G_DATE_FRIDAY:
+			next = G_DATE_SATURDAY;
+			break;
+		case G_DATE_SATURDAY:
+			next = G_DATE_SUNDAY;
+			break;
+		case G_DATE_SUNDAY:
+			next = G_DATE_MONDAY;
+			break;
+		default:
+			next = G_DATE_BAD_WEEKDAY;
+			break;
+	}
+
+	return next;
+}
+
+/**
+ * e_weekday_get_prev:
+ * @weekday: a #GDateWeekday
+ *
+ * Returns the #GDateWeekday before @weekday.
+ *
+ * Returns: the day before @weekday
+ **/
+GDateWeekday
+e_weekday_get_prev (GDateWeekday weekday)
+{
+	GDateWeekday prev;
+
+	/* Verbose for readability. */
+	switch (weekday) {
+		case G_DATE_MONDAY:
+			prev = G_DATE_SUNDAY;
+			break;
+		case G_DATE_TUESDAY:
+			prev = G_DATE_MONDAY;
+			break;
+		case G_DATE_WEDNESDAY:
+			prev = G_DATE_TUESDAY;
+			break;
+		case G_DATE_THURSDAY:
+			prev = G_DATE_WEDNESDAY;
+			break;
+		case G_DATE_FRIDAY:
+			prev = G_DATE_THURSDAY;
+			break;
+		case G_DATE_SATURDAY:
+			prev = G_DATE_FRIDAY;
+			break;
+		case G_DATE_SUNDAY:
+			prev = G_DATE_SATURDAY;
+			break;
+		default:
+			prev = G_DATE_BAD_WEEKDAY;
+			break;
+	}
+
+	return prev;
+}
+
+/**
+ * e_weekday_add_days:
+ * @weekday: a #GDateWeekday
+ * @n_days: number of days to add
+ *
+ * Increments @weekday by @n_days.
+ *
+ * Returns: a #GDateWeekday
+ **/
+GDateWeekday
+e_weekday_add_days (GDateWeekday weekday,
+                    guint n_days)
+{
+	g_return_val_if_fail (
+		g_date_valid_weekday (weekday),
+		G_DATE_BAD_WEEKDAY);
+
+	n_days %= 7;  /* Weekdays repeat every 7 days. */
+
+	while (n_days-- > 0)
+		weekday = e_weekday_get_next (weekday);
+
+	return weekday;
+}
+
+/**
+ * e_weekday_subtract_days:
+ * @weekday: a #GDateWeekday
+ * @n_days: number of days to subtract
+ *
+ * Decrements @weekday by @n_days.
+ *
+ * Returns: a #GDateWeekday
+ **/
+GDateWeekday
+e_weekday_subtract_days (GDateWeekday weekday,
+                         guint n_days)
+{
+	g_return_val_if_fail (
+		g_date_valid_weekday (weekday),
+		G_DATE_BAD_WEEKDAY);
+
+	n_days %= 7;  /* Weekdays repeat every 7 days. */
+
+	while (n_days-- > 0)
+		weekday = e_weekday_get_prev (weekday);
+
+	return weekday;
+}
+
+/**
+ * e_weekday_get_days_between:
+ * @weekday1: a #GDateWeekday
+ * @weekday2: a #GDateWeekday
+ *
+ * Counts the number of days starting at @weekday1 and ending at @weekday2.
+ *
+ * Returns: the number of days
+ **/
+guint
+e_weekday_get_days_between (GDateWeekday weekday1,
+                            GDateWeekday weekday2)
+{
+	guint n_days = 0;
+
+	g_return_val_if_fail (g_date_valid_weekday (weekday1), 0);
+	g_return_val_if_fail (g_date_valid_weekday (weekday2), 0);
+
+	while (weekday1 != weekday2) {
+		n_days++;
+		weekday1 = e_weekday_get_next (weekday1);
+	}
+
+	return n_days;
+}
+
+/**
+ * e_weekday_to_tm_wday:
+ * @weekday: a #GDateWeekday
+ *
+ * Converts a #GDateWeekday to the numbering used in
+ * <structname>struct tm</structname>.
+ *
+ * Returns: number of days since Sunday (0 - 6)
+ **/
+gint
+e_weekday_to_tm_wday (GDateWeekday weekday)
+{
+	gint tm_wday;
+
+	switch (weekday) {
+		case G_DATE_MONDAY:
+			tm_wday = 1;
+			break;
+		case G_DATE_TUESDAY:
+			tm_wday = 2;
+			break;
+		case G_DATE_WEDNESDAY:
+			tm_wday = 3;
+			break;
+		case G_DATE_THURSDAY:
+			tm_wday = 4;
+			break;
+		case G_DATE_FRIDAY:
+			tm_wday = 5;
+			break;
+		case G_DATE_SATURDAY:
+			tm_wday = 6;
+			break;
+		case G_DATE_SUNDAY:
+			tm_wday = 0;
+			break;
+		default:
+			g_return_val_if_reached (-1);
+	}
+
+	return tm_wday;
+}
+
+/**
+ * e_weekday_from_tm_wday:
+ * @tm_wday: number of days since Sunday (0 - 6)
+ *
+ * Converts a weekday in the numbering used in
+ * <structname>struct tm</structname> to a #GDateWeekday.
+ *
+ * Returns: a #GDateWeekday
+ **/
+GDateWeekday
+e_weekday_from_tm_wday (gint tm_wday)
+{
+	GDateWeekday weekday;
+
+	switch (tm_wday) {
+		case 0:
+			weekday = G_DATE_SUNDAY;
+			break;
+		case 1:
+			weekday = G_DATE_MONDAY;
+			break;
+		case 2:
+			weekday = G_DATE_TUESDAY;
+			break;
+		case 3:
+			weekday = G_DATE_WEDNESDAY;
+			break;
+		case 4:
+			weekday = G_DATE_THURSDAY;
+			break;
+		case 5:
+			weekday = G_DATE_FRIDAY;
+			break;
+		case 6:
+			weekday = G_DATE_SATURDAY;
+			break;
+		default:
+			g_return_val_if_reached (G_DATE_BAD_WEEKDAY);
+	}
+
+	return weekday;
+}
+
 /* Evolution Locks for crash recovery */
 static const gchar *
 get_lock_filename (void)
