@@ -113,7 +113,7 @@ emfe_attachment_format (EMailFormatterExtension *extension,
 	gchar *str, *text, *html;
 	gchar *button_id;
 	EAttachmentStore *store;
-	EMailExtensionRegistry *reg;
+	EMailExtensionRegistry *registry;
 	GQueue *extensions;
 	EMailPartAttachment *empa;
 	CamelMimeFilterToHTMLFlags flags;
@@ -156,76 +156,70 @@ emfe_attachment_format (EMailFormatterExtension *extension,
 		}
 	}
 
-        /* If the attachment is requested as RAW, then call the handler directly
-         * and do not append any other code. */
+	registry = e_mail_formatter_get_extension_registry (formatter);
+
+	extensions = e_mail_extension_registry_get_for_mime_type (
+		registry, empa->snoop_mime_type);
+	if (extensions == NULL)
+		extensions = e_mail_extension_registry_get_fallback (
+			registry, empa->snoop_mime_type);
+
+	/* If the attachment is requested as RAW, then call the
+	 * handler directly and do not append any other code. */
 	if ((context->mode == E_MAIL_FORMATTER_MODE_RAW) ||
 	    (context->mode == E_MAIL_FORMATTER_MODE_PRINTING)) {
-		EMailExtensionRegistry *reg;
-		GQueue *extensions;
-		GList *iter;
-		reg = e_mail_formatter_get_extension_registry (formatter);
+		GList *head, *link;
+		gboolean success = FALSE;
 
-		extensions = e_mail_extension_registry_get_for_mime_type (
-					reg, empa->snoop_mime_type);
-		if (!extensions) {
-			extensions = e_mail_extension_registry_get_fallback (
-					reg, empa->snoop_mime_type);
-		}
-
-		if (!extensions)
+		if (extensions == NULL)
 			return FALSE;
 
 		if (context->mode == E_MAIL_FORMATTER_MODE_PRINTING) {
 			gchar *name;
 			EAttachment *attachment;
-			GFileInfo *fi;
-			const gchar *description;
+			GFileInfo *file_info;
+			const gchar *display_name;
+			gchar *description;
 
 			attachment = empa->attachment;
-			fi = e_attachment_get_file_info (attachment);
+			file_info = e_attachment_ref_file_info (attachment);
+			display_name = g_file_info_get_display_name (file_info);
 
-			description = e_attachment_get_description (attachment);
-			if (description && *description) {
+			description = e_attachment_dup_description (attachment);
+			if (description != NULL && *description != '\0') {
 				name = g_strdup_printf (
 					"<h2>Attachment: %s (%s)</h2>\n",
-					description, g_file_info_get_display_name (fi));
+					description, display_name);
 			} else {
 				name = g_strdup_printf (
 					"<h2>Attachment: %s</h2>\n",
-					g_file_info_get_display_name (fi));
+					display_name);
 			}
 
-			camel_stream_write_string (stream, name, cancellable, NULL);
+			camel_stream_write_string (
+				stream, name, cancellable, NULL);
+
+			g_free (description);
 			g_free (name);
+
+			g_object_unref (file_info);
 		}
 
-		for (iter = g_queue_peek_head_link (extensions); iter; iter = iter->next) {
+		head = g_queue_peek_head_link (extensions);
 
-			EMailFormatterExtension *ext;
-			ext = iter->data;
-			if (!ext)
-				continue;
-
-			if (e_mail_formatter_extension_format (ext, formatter,
-				context, part, stream, cancellable)) {
-				return TRUE;
-			}
+		for (link = head; link != NULL; link = g_list_next (link)) {
+			success = e_mail_formatter_extension_format (
+				E_MAIL_FORMATTER_EXTENSION (link->data),
+				formatter, context, part, stream, cancellable);
+			if (success)
+				break;
 		}
 
-		return FALSE;
+		return success;
 	}
 
 	/* E_MAIL_FORMATTER_MODE_NORMAL: */
 
-	reg = e_mail_formatter_get_extension_registry (formatter);
-	extensions = e_mail_extension_registry_get_for_mime_type (
-				reg, empa->snoop_mime_type);
-
-	if (!extensions) {
-		extensions = e_mail_extension_registry_get_fallback (
-				reg, empa->snoop_mime_type);
-	}
-
 	text = e_mail_part_describe (part->part, empa->snoop_mime_type);
 	flags = e_mail_formatter_get_text_format_flags (formatter);
 	html = camel_text_to_html (
@@ -255,13 +249,11 @@ emfe_attachment_format (EMailFormatterExtension *extension,
 	g_free (str);
 	g_free (html);
 
-	if (extensions) {
-		GList *iter;
+	if (extensions != NULL) {
 		CamelStream *content_stream;
-		gboolean ok;
+		gboolean success = FALSE;
 
 		content_stream = camel_stream_mem_new ();
-		ok = FALSE;
 		if (empa->attachment_view_part_id != NULL) {
 			EMailPart *attachment_view_part;
 
@@ -269,8 +261,14 @@ emfe_attachment_format (EMailFormatterExtension *extension,
 				context->part_list,
 				empa->attachment_view_part_id);
 
+			/* Avoid recursion. */
+			if (attachment_view_part == part) {
+				e_mail_part_unref (attachment_view_part);
+				attachment_view_part = NULL;
+			}
+
 			if (attachment_view_part != NULL) {
-				ok = e_mail_formatter_format_as (
+				success = e_mail_formatter_format_as (
 					formatter, context,
 					attachment_view_part,
 					content_stream, NULL,
@@ -279,44 +277,46 @@ emfe_attachment_format (EMailFormatterExtension *extension,
 			}
 
 		} else {
+			GList *head, *link;
 
-			for (iter = g_queue_peek_head_link (extensions); iter; iter = iter->next) {
+			head = g_queue_peek_head_link (extensions);
 
-				EMailFormatterExtension *ext;
-
-				ext = iter->data;
-				if (!ext)
-					continue;
-
-				if (e_mail_formatter_extension_format (
-						ext, formatter, context,
-						part, content_stream,
-						cancellable)) {
-					ok = TRUE;
+			for (link = head; link != NULL; link = g_list_next (link)) {
+				success = e_mail_formatter_extension_format (
+					E_MAIL_FORMATTER_EXTENSION (link->data),
+					formatter, context,
+					part, content_stream,
+					cancellable);
+				if (success)
 					break;
-				}
 			}
 		}
 
-		if (ok) {
+		if (success) {
+			gchar *wrapper_element_id;
+
+			wrapper_element_id = g_strconcat (
+				attachment_part_id, ".wrapper", NULL);
+
 			str = g_strdup_printf (
 				"<tr><td colspan=\"2\">"
 				"<div class=\"attachment-wrapper\" id=\"%s\">",
-				attachment_part_id);
+				wrapper_element_id);
 
 			camel_stream_write_string (
 				stream, str, cancellable, NULL);
-			g_free (str);
 
 			g_seekable_seek (
 				G_SEEKABLE (content_stream), 0,
 				G_SEEK_SET, cancellable, NULL);
 			camel_stream_write_to_stream (
-					content_stream, stream,
-					cancellable, NULL);
+				content_stream, stream, cancellable, NULL);
 
 			camel_stream_write_string (
 				stream, "</div></td></tr>", cancellable, NULL);
+
+			g_free (wrapper_element_id);
+			g_free (str);
 		}
 
 		g_object_unref (content_stream);
