@@ -34,21 +34,27 @@
 #include "e-mail-printer.h"
 #include "e-mail-display.h"
 
-enum {
-        BUTTON_SELECT_ALL,
-        BUTTON_SELECT_NONE,
-        BUTTON_TOP,
-        BUTTON_UP,
-        BUTTON_DOWN,
-        BUTTON_BOTTOM,
-        BUTTONS_COUNT
-};
-
 #define w(x)
 
+#define E_MAIL_PRINTER_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), E_TYPE_MAIL_PRINTER, EMailPrinterPrivate))
+
+enum {
+	BUTTON_SELECT_ALL,
+	BUTTON_SELECT_NONE,
+	BUTTON_TOP,
+	BUTTON_UP,
+	BUTTON_DOWN,
+	BUTTON_BOTTOM,
+	BUTTONS_COUNT
+};
+
+typedef struct _AsyncContext AsyncContext;
+
 struct _EMailPrinterPrivate {
-	EMailFormatterPrint *formatter;
-	EMailPartList *parts_list;
+	EMailFormatter *formatter;
+	EMailPartList *part_list;
 
 	gchar *export_filename;
 
@@ -63,21 +69,22 @@ struct _EMailPrinterPrivate {
 	GtkPrintOperationAction print_action;
 };
 
-G_DEFINE_TYPE (
-	EMailPrinter,
-	e_mail_printer,
-	G_TYPE_OBJECT);
+struct _AsyncContext {
+	WebKitWebView *web_view;
+	gulong load_status_handler_id;
+
+	GCancellable *cancellable;
+	GMainContext *main_context;
+
+	GtkPrintOperationAction print_action;
+	GtkPrintOperationResult print_result;
+};
 
 enum {
 	PROP_0,
 	PROP_PART_LIST
 };
 
-enum {
-	SIGNAL_DONE,
-	LAST_SIGNAL
-};
-
 enum {
 	COLUMN_ACTIVE,
 	COLUMN_HEADER_NAME,
@@ -86,11 +93,30 @@ enum {
 	LAST_COLUMN
 };
 
-static guint signals[LAST_SIGNAL];
+G_DEFINE_TYPE (
+	EMailPrinter,
+	e_mail_printer,
+	G_TYPE_OBJECT);
+
+static void
+async_context_free (AsyncContext *async_context)
+{
+	if (async_context->load_status_handler_id > 0)
+		g_signal_handler_disconnect (
+			async_context->web_view,
+			async_context->load_status_handler_id);
+
+	g_clear_object (&async_context->web_view);
+	g_clear_object (&async_context->cancellable);
+
+	g_main_context_unref (async_context->main_context);
+
+	g_slice_free (AsyncContext, async_context);
+}
 
 static gint
-emp_header_name_equal (const EMailFormatterHeader *h1,
-                       const EMailFormatterHeader *h2)
+mail_printer_header_name_equal (const EMailFormatterHeader *h1,
+                                const EMailFormatterHeader *h2)
 {
 	if ((h2->value == NULL) || (h1->value == NULL)) {
 		return g_strcmp0 (h1->name, h2->name);
@@ -104,9 +130,9 @@ emp_header_name_equal (const EMailFormatterHeader *h1,
 }
 
 static void
-emp_draw_footer (GtkPrintOperation *operation,
-                 GtkPrintContext *context,
-                 gint page_nr)
+mail_printer_draw_footer_cb (GtkPrintOperation *operation,
+                             GtkPrintContext *context,
+                             gint page_nr)
 {
 	PangoFontDescription *desc;
 	PangoLayout *layout;
@@ -140,131 +166,6 @@ emp_draw_footer (GtkPrintOperation *operation,
 	g_free (text);
 }
 
-static void
-emp_printing_done (GtkPrintOperation *operation,
-                   GtkPrintOperationResult result,
-                   gpointer user_data)
-{
-	EMailPrinter *emp = user_data;
-
-	g_signal_emit (emp, signals[SIGNAL_DONE], 0, operation, result);
-}
-
-static gboolean
-do_run_print_operation (EMailPrinter *emp)
-{
-	WebKitWebFrame *frame;
-
-	frame = webkit_web_view_get_main_frame (emp->priv->webview);
-
-	webkit_web_frame_print_full (
-		frame, emp->priv->operation, emp->priv->print_action, NULL);
-
-	return FALSE;
-}
-
-static void
-emp_start_printing (GObject *object,
-                    GParamSpec *pspec,
-                    gpointer user_data)
-{
-	WebKitWebView *web_view;
-	WebKitLoadStatus load_status;
-	EMailPrinter *emp = user_data;
-
-	web_view = WEBKIT_WEB_VIEW (object);
-	load_status = webkit_web_view_get_load_status (web_view);
-
-	if (load_status != WEBKIT_LOAD_FINISHED)
-		return;
-
-	/* WebKit reloads the page once more right before starting to print, so
-	 * disconnect this handler after the first time, so that we don't start
-	 * another printing operation */
-	g_signal_handlers_disconnect_by_func (
-		object, emp_start_printing, user_data);
-
-	if (emp->priv->print_action == GTK_PRINT_OPERATION_ACTION_EXPORT) {
-		gtk_print_operation_set_export_filename (
-			emp->priv->operation, emp->priv->export_filename);
-	}
-
-	/* Give WebKit some time to perform layouting and rendering before
-	 * we start printing. 500ms should be enough in most cases... */
-	g_timeout_add_full (
-		G_PRIORITY_DEFAULT, 500,
-		(GSourceFunc) do_run_print_operation,
-		g_object_ref (emp), g_object_unref);
-}
-
-static void
-emp_run_print_operation (EMailPrinter *emp,
-                         EMailFormatter *formatter)
-{
-	EMailPartList *part_list;
-	CamelFolder *folder;
-	const gchar *message_uid;
-	const gchar *default_charset, *charset;
-	gchar *mail_uri;
-
-	part_list = emp->priv->parts_list;
-	folder = e_mail_part_list_get_folder (part_list);
-	message_uid = e_mail_part_list_get_message_uid (part_list);
-	default_charset = formatter ? e_mail_formatter_get_default_charset (formatter) : NULL;
-	charset = formatter ? e_mail_formatter_get_charset (formatter) : NULL;
-
-	if (!default_charset)
-		default_charset = "";
-	if (!charset)
-		charset = "";
-
-	mail_uri = e_mail_part_build_uri (
-		folder, message_uid,
-		"__evo-load-image", G_TYPE_BOOLEAN, TRUE,
-		"mode", G_TYPE_INT, E_MAIL_FORMATTER_MODE_PRINTING,
-		"formatter_default_charset", G_TYPE_STRING, default_charset,
-		"formatter_charset", G_TYPE_STRING, charset,
-		NULL);
-
-	/* Print_layout is a special EMPart created by EMFormatHTMLPrint */
-	if (emp->priv->webview == NULL) {
-		EMailFormatter *emp_formatter;
-
-		emp->priv->webview = g_object_new (
-			E_TYPE_MAIL_DISPLAY,
-			"mode", E_MAIL_FORMATTER_MODE_PRINTING, NULL);
-		e_web_view_set_enable_frame_flattening (E_WEB_VIEW (emp->priv->webview), FALSE);
-		e_mail_display_set_force_load_images (
-			E_MAIL_DISPLAY (emp->priv->webview), TRUE);
-
-		emp_formatter = e_mail_display_get_formatter (E_MAIL_DISPLAY (emp->priv->webview));
-		if (default_charset && *default_charset)
-			e_mail_formatter_set_default_charset (emp_formatter, default_charset);
-		if (charset && *charset)
-			e_mail_formatter_set_charset (emp_formatter, charset);
-
-		g_object_ref_sink (emp->priv->webview);
-		g_signal_connect (
-			emp->priv->webview, "notify::load-status",
-			G_CALLBACK (emp_start_printing), emp);
-
-		w ({
-			GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-			GtkWidget *sw = gtk_scrolled_window_new (NULL, NULL);
-			gtk_container_add (GTK_CONTAINER (window), sw);
-			gtk_container_add (
-				GTK_CONTAINER (sw),
-				GTK_WIDGET (emp->priv->webview));
-			gtk_widget_show_all (window);
-		});
-	}
-	e_mail_display_set_parts_list (
-		E_MAIL_DISPLAY (emp->priv->webview), emp->priv->parts_list);
-	webkit_web_view_load_uri (emp->priv->webview, mail_uri);
-
-	g_free (mail_uri);
-}
-
 static void
 set_header_visible (EMailPrinter *emp,
                     EMailFormatterHeader *header,
@@ -642,9 +543,162 @@ emp_create_headers_tab (GtkPrintOperation *operation,
 	return hbox;
 }
 
+static gboolean
+mail_printer_print_timeout_cb (gpointer user_data)
+{
+	GSimpleAsyncResult *simple;
+	AsyncContext *async_context;
+	GCancellable *cancellable;
+	GtkPrintOperation *print_operation;
+	GtkPrintOperationAction print_action;
+	EMailPrinter *printer;
+	WebKitWebFrame *web_frame;
+	gulong create_custom_widget_handler_id;
+	gulong draw_page_handler_id;
+	GError *error = NULL;
+
+	simple = G_SIMPLE_ASYNC_RESULT (user_data);
+	async_context = g_simple_async_result_get_op_res_gpointer (simple);
+
+	cancellable = async_context->cancellable;
+	print_action = async_context->print_action;
+
+	/* Check for cancellation one last time before printing. */
+	if (g_cancellable_set_error_if_cancelled (cancellable, &error))
+		goto exit;
+
+	/* This returns a new reference. */
+	printer = (EMailPrinter *) g_async_result_get_source_object (
+		G_ASYNC_RESULT (simple));
+
+	print_operation = e_print_operation_new ();
+	gtk_print_operation_set_show_progress (print_operation, TRUE);
+	gtk_print_operation_set_unit (print_operation, GTK_UNIT_PIXEL);
+
+	if (async_context->print_action == GTK_PRINT_OPERATION_ACTION_EXPORT) {
+		const gchar *export_filename;
+
+		export_filename =
+			e_mail_printer_get_export_filename (printer);
+		gtk_print_operation_set_export_filename (
+			print_operation, export_filename);
+	}
+
+	create_custom_widget_handler_id = g_signal_connect (
+		print_operation, "create-custom-widget",
+		G_CALLBACK (emp_create_headers_tab), printer);
+
+	draw_page_handler_id = g_signal_connect (
+		print_operation, "draw-page",
+		G_CALLBACK (mail_printer_draw_footer_cb),
+		async_context->cancellable);
+
+	web_frame = webkit_web_view_get_main_frame (async_context->web_view);
+
+	async_context->print_result = webkit_web_frame_print_full (
+		web_frame, print_operation, print_action, &error);
+
+	/* Sanity check. */
+	switch (async_context->print_result) {
+		case GTK_PRINT_OPERATION_RESULT_ERROR:
+			if (error == NULL)
+				g_warning (
+					"WebKit print operation returned "
+					"ERROR result without setting a "
+					"GError");
+			break;
+		case GTK_PRINT_OPERATION_RESULT_APPLY:
+			if (error != NULL)
+				g_warning (
+					"WebKit print operation returned "
+					"APPLY result but also set a GError");
+			break;
+		case GTK_PRINT_OPERATION_RESULT_CANCEL:
+			if (error != NULL)
+				g_warning (
+					"WebKit print operation returned "
+					"CANCEL result but also set a GError");
+			break;
+		default:
+			g_warn_if_reached ();
+	}
+
+	g_signal_handler_disconnect (
+		print_operation, create_custom_widget_handler_id);
+
+	g_signal_handler_disconnect (
+		print_operation, draw_page_handler_id);
+
+	g_object_unref (print_operation);
+
+	g_object_unref (printer);
+
+exit:
+	if (error != NULL)
+		g_simple_async_result_take_error (simple, error);
+
+	g_simple_async_result_complete_in_idle (simple);
+
+	return FALSE;
+}
+
 static void
-emp_set_parts_list (EMailPrinter *emp,
-                    EMailPartList *parts_list)
+mail_printer_load_status_cb (WebKitWebView *web_view,
+                             GParamSpec *pspec,
+                             GSimpleAsyncResult *simple)
+{
+	AsyncContext *async_context;
+	WebKitLoadStatus load_status;
+	GCancellable *cancellable;
+	GError *error = NULL;
+
+	/* Note: we disregard WEBKIT_LOAD_FAILED and print what we can. */
+	load_status = webkit_web_view_get_load_status (web_view);
+	if (load_status != WEBKIT_LOAD_FINISHED)
+		return;
+
+	/* Signal handlers are holding the only GSimpleAsyncResult
+	 * references.  This is to avoid finalizing it prematurely. */
+	g_object_ref (simple);
+
+	async_context = g_simple_async_result_get_op_res_gpointer (simple);
+	cancellable = async_context->cancellable;
+
+	/* WebKit reloads the page once more right before starting to print,
+	 * so disconnect this handler after the first time to avoid starting
+	 * another print operation. */
+	g_signal_handler_disconnect (
+		async_context->web_view,
+		async_context->load_status_handler_id);
+	async_context->load_status_handler_id = 0;
+
+	/* Check if we've been cancelled. */
+	if (g_cancellable_set_error_if_cancelled (cancellable, &error)) {
+		g_simple_async_result_take_error (simple, error);
+		g_simple_async_result_complete_in_idle (simple);
+
+	/* Give WebKit some time to perform layouting and rendering before
+	 * we start printing.  500ms should be enough in most cases. */
+	} else {
+		GSource *timeout_source;
+
+		timeout_source = g_timeout_source_new (500);
+		g_source_set_callback (
+			timeout_source,
+			mail_printer_print_timeout_cb,
+			g_object_ref (simple),
+			(GDestroyNotify) g_object_unref);
+		g_source_attach (
+			timeout_source, async_context->main_context);
+		g_source_unref (timeout_source);
+	}
+
+	g_object_unref (simple);
+}
+
+static void
+mail_printer_build_model (EMailPrinter *printer,
+                          EMailPartList *part_list)
 {
 	CamelMediumHeader *header;
 	CamelMimeMessage *message;
@@ -653,22 +707,22 @@ emp_set_parts_list (EMailPrinter *emp,
 	gint i;
 	GtkTreeIter last_known = { 0 };
 
-	g_return_if_fail (parts_list);
-
-	emp->priv->parts_list = g_object_ref (parts_list);
-
-	if (emp->priv->headers)
-		g_object_unref (emp->priv->headers);
-	emp->priv->headers = gtk_list_store_new (
+	if (printer->priv->headers != NULL)
+		g_object_unref (printer->priv->headers);
+	printer->priv->headers = gtk_list_store_new (
 		5,
-		G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_INT);
+		G_TYPE_BOOLEAN,  /* COLUMN_ACTIVE */
+		G_TYPE_STRING,   /* COLUMN_HEADER_NAME */
+		G_TYPE_STRING,   /* COLUMN_HEADER_VALUE */
+		G_TYPE_POINTER,  /* COLUMN_HEADER_STRUCT */
+		G_TYPE_INT);     /* ??? */
 
-	message = e_mail_part_list_get_message (parts_list);
+	message = e_mail_part_list_get_message (part_list);
 	headers = camel_medium_get_headers (CAMEL_MEDIUM (message));
 	if (!headers)
 		return;
 
-	headers_queue = e_mail_formatter_dup_headers (E_MAIL_FORMATTER (emp->priv->formatter));
+	headers_queue = e_mail_formatter_dup_headers (printer->priv->formatter);
 	for (i = 0; i < headers->len; i++) {
 		GtkTreeIter iter;
 		GList *found_header;
@@ -677,46 +731,82 @@ emp_set_parts_list (EMailPrinter *emp,
 		header = &g_array_index (headers, CamelMediumHeader, i);
 		emfh = e_mail_formatter_header_new (header->name, header->value);
 
-		found_header = g_queue_find_custom (headers_queue, emfh, (GCompareFunc) emp_header_name_equal);
+		found_header = g_queue_find_custom (
+			headers_queue, emfh, (GCompareFunc)
+			mail_printer_header_name_equal);
 
 		if (!found_header) {
 			emfh->flags |= E_MAIL_FORMATTER_HEADER_FLAG_HIDDEN;
 			e_mail_formatter_add_header_struct (
-				E_MAIL_FORMATTER (emp->priv->formatter), emfh);
-			gtk_list_store_append (emp->priv->headers, &iter);
+				printer->priv->formatter, emfh);
+			gtk_list_store_append (printer->priv->headers, &iter);
 		} else {
-			if (gtk_list_store_iter_is_valid (emp->priv->headers, &last_known))
-				gtk_list_store_insert_after (emp->priv->headers, &iter, &last_known);
+			if (gtk_list_store_iter_is_valid (printer->priv->headers, &last_known))
+				gtk_list_store_insert_after (printer->priv->headers, &iter, &last_known);
 			else
-				gtk_list_store_insert_after (emp->priv->headers, &iter, NULL);
+				gtk_list_store_insert_after (printer->priv->headers, &iter, NULL);
 
 			last_known = iter;
 		}
 
 		gtk_list_store_set (
-			emp->priv->headers, &iter,
+			printer->priv->headers, &iter,
 			COLUMN_ACTIVE, (found_header != NULL),
 			COLUMN_HEADER_NAME, emfh->name,
 			COLUMN_HEADER_VALUE, emfh->value,
 			COLUMN_HEADER_STRUCT, emfh, -1);
 	}
 
-	g_queue_free_full (headers_queue, (GDestroyNotify) e_mail_formatter_header_free);
+	g_queue_free_full (
+		headers_queue,
+		(GDestroyNotify) e_mail_formatter_header_free);
+
 	camel_medium_free_headers (CAMEL_MEDIUM (message), headers);
 }
 
-static void
-emp_set_property (GObject *object,
-                  guint property_id,
-                  const GValue *value,
-                  GParamSpec *pspec)
+static WebKitWebView *
+mail_printer_new_web_view (const gchar *charset,
+                           const gchar *default_charset)
 {
-	EMailPrinter *emp = E_MAIL_PRINTER (object);
+	WebKitWebView *web_view;
+	EMailFormatter *formatter;
 
+	web_view = g_object_new (
+		E_TYPE_MAIL_DISPLAY,
+		"mode", E_MAIL_FORMATTER_MODE_PRINTING, NULL);
+	e_web_view_set_enable_frame_flattening (E_WEB_VIEW (web_view), FALSE);
+	e_mail_display_set_force_load_images (E_MAIL_DISPLAY (web_view), TRUE);
+
+	formatter = e_mail_display_get_formatter (E_MAIL_DISPLAY (web_view));
+	if (charset != NULL && *charset != '\0')
+		e_mail_formatter_set_charset (formatter, charset);
+	if (default_charset != NULL && *default_charset != '\0')
+		e_mail_formatter_set_default_charset (formatter, default_charset);
+
+	return web_view;
+}
+
+static void
+mail_printer_set_part_list (EMailPrinter *printer,
+                            EMailPartList *part_list)
+{
+	g_return_if_fail (E_IS_MAIL_PART_LIST (part_list));
+	g_return_if_fail (printer->priv->part_list == NULL);
+
+	printer->priv->part_list = g_object_ref (part_list);
+}
+
+static void
+mail_printer_set_property (GObject *object,
+                           guint property_id,
+                           const GValue *value,
+                           GParamSpec *pspec)
+{
 	switch (property_id) {
-
 		case PROP_PART_LIST:
-			emp_set_parts_list (emp, g_value_get_pointer (value));
+			mail_printer_set_part_list (
+				E_MAIL_PRINTER (object),
+				g_value_get_object (value));
 			return;
 	}
 
@@ -724,17 +814,17 @@ emp_set_property (GObject *object,
 }
 
 static void
-emp_get_property (GObject *object,
-                  guint property_id,
-                  GValue *value,
-                  GParamSpec *pspec)
+mail_printer_get_property (GObject *object,
+                           guint property_id,
+                           GValue *value,
+                           GParamSpec *pspec)
 {
-	EMailPrinter *emp = E_MAIL_PRINTER (object);
-
 	switch (property_id) {
-
 		case PROP_PART_LIST:
-			g_value_set_pointer (value, emp->priv->parts_list);
+			g_value_take_object (
+				value,
+				e_mail_printer_ref_part_list (
+				E_MAIL_PRINTER (object)));
 			return;
 	}
 
@@ -742,55 +832,71 @@ emp_get_property (GObject *object,
 }
 
 static void
-emp_finalize (GObject *object)
+mail_printer_dispose (GObject *object)
 {
-	EMailPrinterPrivate *priv = E_MAIL_PRINTER (object)->priv;
+	EMailPrinterPrivate *priv;
 
-	if (priv->formatter) {
-		g_object_unref (priv->formatter);
-		priv->formatter = NULL;
-	}
+	priv = E_MAIL_PRINTER_GET_PRIVATE (object);
 
-	if (priv->headers) {
+	if (priv->headers != NULL) {
+		GtkTreeModel *model;
 		GtkTreeIter iter;
+		gboolean valid;
 
-		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (priv->headers), &iter)) {
-			do {
-				EMailFormatterHeader *header = NULL;
-				gtk_tree_model_get (
-					GTK_TREE_MODEL (priv->headers), &iter,
-					COLUMN_HEADER_STRUCT, &header, -1);
-				e_mail_formatter_header_free (header);
-			} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (priv->headers), &iter));
+		model = GTK_TREE_MODEL (priv->headers);
+
+		valid = gtk_tree_model_get_iter_first (model, &iter);
+		while (valid) {
+			EMailFormatterHeader *header = NULL;
+
+			gtk_tree_model_get (
+				model, &iter,
+				COLUMN_HEADER_STRUCT, &header, -1);
+			e_mail_formatter_header_free (header);
+
+			valid = gtk_tree_model_iter_next (model, &iter);
 		}
-		g_object_unref (priv->headers);
-		priv->headers = NULL;
 	}
 
-	if (priv->webview) {
-		g_object_unref (priv->webview);
-		priv->webview = NULL;
-	}
+	g_clear_object (&priv->formatter);
+	g_clear_object (&priv->part_list);
+	g_clear_object (&priv->headers);
+	g_clear_object (&priv->webview);
+	g_clear_object (&priv->operation);
 
-	if (priv->uri) {
-		g_free (priv->uri);
-		priv->uri = NULL;
-	}
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (e_mail_printer_parent_class)->dispose (object);
+}
 
-	if (priv->operation) {
-		g_object_unref (priv->operation);
-		priv->operation = NULL;
-	}
+static void
+mail_printer_finalize (GObject *object)
+{
+	EMailPrinterPrivate *priv;
 
-	if (priv->parts_list) {
-		g_object_unref (priv->parts_list);
-		priv->parts_list = NULL;
-	}
+	priv = E_MAIL_PRINTER_GET_PRIVATE (object);
 
-        /* Chain up to parent's finalize() method. */
+	g_free (priv->uri);
+
+	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_mail_printer_parent_class)->finalize (object);
 }
 
+static void
+mail_printer_constructed (GObject *object)
+{
+	EMailPrinter *printer;
+	EMailPartList *part_list;
+
+	printer = E_MAIL_PRINTER (object);
+
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (e_mail_printer_parent_class)->constructed (object);
+
+	part_list = e_mail_printer_ref_part_list (printer);
+	mail_printer_build_model (printer, part_list);
+	g_object_unref (part_list);
+}
+
 static void
 e_mail_printer_class_init (EMailPrinterClass *class)
 {
@@ -799,85 +905,157 @@ e_mail_printer_class_init (EMailPrinterClass *class)
 	g_type_class_add_private (class, sizeof (EMailPrinterPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
-	object_class->set_property = emp_set_property;
-	object_class->get_property = emp_get_property;
-	object_class->finalize = emp_finalize;
+	object_class->set_property = mail_printer_set_property;
+	object_class->get_property = mail_printer_get_property;
+	object_class->dispose = mail_printer_dispose;
+	object_class->finalize = mail_printer_finalize;
+	object_class->constructed = mail_printer_constructed;
 
 	g_object_class_install_property (
 		object_class,
 		PROP_PART_LIST,
-		g_param_spec_pointer (
-			"parts-list",
-			"Parts List",
+		g_param_spec_object (
+			"part-list",
+			"Part List",
 			NULL,
+			E_TYPE_MAIL_PART_LIST,
 			G_PARAM_READWRITE |
 			G_PARAM_CONSTRUCT_ONLY));
-
-	signals[SIGNAL_DONE] = g_signal_new (
-		"done",
-		G_TYPE_FROM_CLASS (class),
-		G_SIGNAL_RUN_FIRST,
-		G_STRUCT_OFFSET (EMailPrinterClass, done),
-		NULL, NULL,
-		e_marshal_VOID__OBJECT_INT,
-		G_TYPE_NONE, 2,
-		GTK_TYPE_PRINT_OPERATION, G_TYPE_INT);
 }
 
 static void
-e_mail_printer_init (EMailPrinter *emp)
+e_mail_printer_init (EMailPrinter *printer)
 {
-	emp->priv = G_TYPE_INSTANCE_GET_PRIVATE (
-		emp, E_TYPE_MAIL_PRINTER, EMailPrinterPrivate);
+	printer->priv = E_MAIL_PRINTER_GET_PRIVATE (printer);
 
-	emp->priv->formatter = (EMailFormatterPrint *) e_mail_formatter_print_new ();
-	emp->priv->headers = NULL;
-	emp->priv->webview = NULL;
+	printer->priv->formatter = e_mail_formatter_print_new ();
 }
 
 EMailPrinter *
-e_mail_printer_new (EMailPartList *source)
+e_mail_printer_new (EMailPartList *part_list)
 {
-	EMailPrinter *emp;
+	g_return_val_if_fail (E_IS_MAIL_PART_LIST (part_list), NULL);
 
-	emp = g_object_new (
+	return g_object_new (
 		E_TYPE_MAIL_PRINTER,
-		"parts-list", source, NULL);
+		"part-list", part_list, NULL);
+}
 
-	return emp;
+EMailPartList *
+e_mail_printer_ref_part_list (EMailPrinter *printer)
+{
+	g_return_val_if_fail (E_IS_MAIL_PRINTER (printer), NULL);
+
+	return g_object_ref (printer->priv->part_list);
 }
 
 void
-e_mail_printer_print (EMailPrinter *emp,
+e_mail_printer_print (EMailPrinter *printer,
                       GtkPrintOperationAction action,
                       EMailFormatter *formatter,
-                      GCancellable *cancellable)
+                      GCancellable *cancellable,
+                      GAsyncReadyCallback callback,
+                      gpointer user_data)
 {
-	g_return_if_fail (E_IS_MAIL_PRINTER (emp));
+	GSimpleAsyncResult *simple;
+	AsyncContext *async_context;
+	WebKitWebView *web_view;
+	EMailPartList *part_list;
+	CamelFolder *folder;
+	const gchar *message_uid;
+	const gchar *charset = NULL;
+	const gchar *default_charset = NULL;
+	gchar *mail_uri;
+	gulong handler_id;
 
-	if (emp->priv->operation)
-		g_object_unref (emp->priv->operation);
-	emp->priv->operation = e_print_operation_new ();
-	emp->priv->print_action = action;
-	gtk_print_operation_set_unit (emp->priv->operation, GTK_UNIT_PIXEL);
+	g_return_if_fail (E_IS_MAIL_PRINTER (printer));
+	/* EMailFormatter can be NULL. */
 
-	gtk_print_operation_set_show_progress (emp->priv->operation, TRUE);
-	g_signal_connect (
-		emp->priv->operation, "create-custom-widget",
-		G_CALLBACK (emp_create_headers_tab), emp);
-	g_signal_connect (
-		emp->priv->operation, "done",
-		G_CALLBACK (emp_printing_done), emp);
-	g_signal_connect (
-		emp->priv->operation, "draw-page",
-		G_CALLBACK (emp_draw_footer), NULL);
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->print_action = action;
+	async_context->main_context = g_main_context_ref_thread_default ();
 
-	if (cancellable)
-		g_signal_connect_swapped (
-			cancellable, "cancelled",
-			G_CALLBACK (gtk_print_operation_cancel), emp->priv->operation);
+	if (G_IS_CANCELLABLE (cancellable))
+		async_context->cancellable = g_object_ref (cancellable);
 
-	emp_run_print_operation (emp, formatter);
+	part_list = e_mail_printer_ref_part_list (printer);
+	folder = e_mail_part_list_get_folder (part_list);
+	message_uid = e_mail_part_list_get_message_uid (part_list);
+
+	if (formatter != NULL) {
+		charset =
+			e_mail_formatter_get_charset (formatter);
+		default_charset =
+			e_mail_formatter_get_default_charset (formatter);
+	}
+
+	if (charset == NULL)
+		charset = "";
+	if (default_charset == NULL)
+		default_charset = "";
+
+	simple = g_simple_async_result_new (
+		G_OBJECT (printer), callback,
+		user_data, e_mail_printer_print);
+
+	g_simple_async_result_set_check_cancellable (simple, cancellable);
+
+	g_simple_async_result_set_op_res_gpointer (
+		simple, async_context, (GDestroyNotify) async_context_free);
+
+	web_view = mail_printer_new_web_view (charset, default_charset);
+	e_mail_display_set_parts_list (E_MAIL_DISPLAY (web_view), part_list);
+
+	async_context->web_view = g_object_ref_sink (web_view);
+
+	handler_id = g_signal_connect_data (
+		web_view, "notify::load-status",
+		G_CALLBACK (mail_printer_load_status_cb),
+		g_object_ref (simple),
+		(GClosureNotify) g_object_unref, 0);
+	async_context->load_status_handler_id = handler_id;
+
+	mail_uri = e_mail_part_build_uri (
+		folder, message_uid,
+		"__evo-load-image", G_TYPE_BOOLEAN, TRUE,
+		"mode", G_TYPE_INT, E_MAIL_FORMATTER_MODE_PRINTING,
+		"formatter_default_charset", G_TYPE_STRING, default_charset,
+		"formatter_charset", G_TYPE_STRING, charset,
+		NULL);
+
+	webkit_web_view_load_uri (web_view, mail_uri);
+
+	g_free (mail_uri);
+
+	g_object_unref (simple);
+
+	g_object_unref (part_list);
+}
+
+GtkPrintOperationResult
+e_mail_printer_print_finish (EMailPrinter *printer,
+                             GAsyncResult *result,
+                             GError **error)
+{
+	GSimpleAsyncResult *simple;
+	AsyncContext *async_context;
+
+	g_return_val_if_fail (
+		g_simple_async_result_is_valid (
+		result, G_OBJECT (printer), e_mail_printer_print),
+		GTK_PRINT_OPERATION_RESULT_ERROR);
+
+	simple = G_SIMPLE_ASYNC_RESULT (result);
+	async_context = g_simple_async_result_get_op_res_gpointer (simple);
+
+	if (g_simple_async_result_propagate_error (simple, error))
+		return GTK_PRINT_OPERATION_RESULT_ERROR;
+
+	g_warn_if_fail (
+		async_context->print_result !=
+		GTK_PRINT_OPERATION_RESULT_ERROR);
+
+	return async_context->print_result;
 }
 
 const gchar *
@@ -894,8 +1072,6 @@ e_mail_printer_set_export_filename (EMailPrinter *printer,
 {
 	g_return_if_fail (E_IS_MAIL_PRINTER (printer));
 
-	if (printer->priv->export_filename)
-	  g_free (printer->priv->export_filename);
-
+	g_free (printer->priv->export_filename);
 	printer->priv->export_filename = g_strdup (filename);
 }
