@@ -385,6 +385,10 @@ cal_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 	shell = e_shell_window_get_shell (shell_window);
 	shell_settings = e_shell_get_shell_settings (shell);
 
+	g_signal_handlers_disconnect_by_func (
+		shell_window,
+		cal_shell_sidebar_restore_state_cb, shell_sidebar);
+
 	selector = E_SOURCE_SELECTOR (priv->selector);
 	registry = e_source_selector_get_registry (selector);
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (selector));
@@ -403,6 +407,12 @@ cal_shell_sidebar_restore_state_cb (EShellWindow *shell_window,
 		G_CALLBACK (cal_shell_sidebar_primary_selection_changed_cb),
 		shell_sidebar);
 
+	/* This will trigger our "row-changed" signal handler for each
+	 * calendar source, so the appropriate ECalClients get added to
+	 * the ECalModel, which will then create view objects to display
+	 * the calendar content.  This all happens asynchronously. */
+	e_source_selector_update_all_rows (selector);
+
 	/* Bind GObject properties to settings keys. */
 
 	settings = g_settings_new ("org.gnome.evolution.calendar");
