@@ -300,6 +300,16 @@ task_shell_content_restore_state_cb (EShellWindow *shell_window,
 	g_object_unref (settings);
 }
 
+static void
+task_shell_content_is_editing_changed_cb (ETaskTable *task_table,
+                                          GParamSpec *param,
+                                          EShellView *shell_view)
+{
+	g_return_if_fail (E_IS_SHELL_VIEW (shell_view));
+
+	e_shell_view_update_actions (shell_view);
+}
+
 static GtkOrientation
 task_shell_content_get_orientation (ETaskShellContent *task_shell_content)
 {
@@ -542,6 +552,10 @@ task_shell_content_constructed (GObject *object)
 		G_CALLBACK (task_shell_content_selection_change_cb),
 		object);
 
+	g_signal_connect (
+		priv->task_table, "notify::is-editing",
+		G_CALLBACK (task_shell_content_is_editing_changed_cb), shell_view);
+
 	g_signal_connect_swapped (
 		priv->task_model, "model-row-changed",
 		G_CALLBACK (task_shell_content_model_row_changed_cb),
