@@ -103,7 +103,11 @@ make_part_attachment (EMailParser *parser,
                       GCancellable *cancellable,
                       GQueue *out_mail_parts)
 {
-	if (camel_content_type_is (camel_mime_part_get_content_type (part), "text", "html")) {
+	CamelContentType *ct;
+
+	ct = camel_mime_part_get_content_type (part);
+
+	if (camel_content_type_is (ct, "text", "html")) {
 		GQueue work_queue = G_QUEUE_INIT;
 		EMailPart *mail_part;
 		gint len;
@@ -111,10 +115,12 @@ make_part_attachment (EMailParser *parser,
 		/* always show HTML as attachments and not inline */
 		camel_mime_part_set_disposition (part, "attachment");
 
-		if (!camel_mime_part_get_filename (part)) {
-			gchar *str = g_strdup_printf ("%s.html", _("attachment"));
-			camel_mime_part_set_filename (part, str);
-			g_free (str);
+		if (camel_mime_part_get_filename (part) == NULL) {
+			gchar *filename;
+
+			filename = g_strdup_printf ("%s.html", _("attachment"));
+			camel_mime_part_set_filename (part, filename);
+			g_free (filename);
 		}
 
 		len = part_id->len;
@@ -131,8 +137,8 @@ make_part_attachment (EMailParser *parser,
 		e_queue_transfer (&work_queue, out_mail_parts);
 
 	} else if (force_html && CAMEL_IS_MIME_MESSAGE (part)) {
-		/* message was asked to be formatted as text/html;
-		 * might be for cases where message itself is a text/html part */
+		/* Note, the message was asked to be formatted as
+		 * text/html; but the message may already be text/html. */
 		CamelMimePart *new_part;
 		CamelDataWrapper *content;
 
@@ -180,27 +186,30 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 	gint i, nparts, partidlen;
 	CamelContentType *ct;
 	gboolean has_calendar = FALSE;
+	gboolean has_html = FALSE;
+	gboolean prefer_html;
 	GQueue plain_text_parts = G_QUEUE_INIT;
 	GQueue work_queue = G_QUEUE_INIT;
 
 	emp_pp = (EMailParserPreferPlain *) extension;
+	prefer_html = (emp_pp->mode == PREFER_HTML);
 
 	ct = camel_mime_part_get_content_type (part);
 
-	/* We can actually parse HTML, but just to discard it when
-	 * "Only ever show plain text" mode is set */
+	/* We can actually parse HTML, but just discard it
+	 * when "Only ever show plain text" mode is set. */
 	if (camel_content_type_is (ct, "text", "html")) {
 
 		/* Prevent recursion, fall back to next (real text/html) parser */
 		if (strstr (part_id->str, ".alternative-prefer-plain.") != NULL)
 			return FALSE;
 
-		/* Not enforcing text/plain, so use real parser */
+		/* Not enforcing text/plain, so use real parser. */
 		if (emp_pp->mode != ONLY_PLAIN)
 			return FALSE;
 
-		/* Enforcing text/plain, but got only HTML part, so add it
-		 * as attachment, to not show empty message preview, which
+		/* Enforcing text/plain but got only HTML part, so add it
+		 * as attachment to not show empty message preview, which
 		 * is confusing. */
 		make_part_attachment (
 			parser, part, part_id, FALSE,
@@ -230,18 +239,18 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 		g_string_append_printf (part_id, ".alternative-prefer-plain.%d", i);
 
 		if (camel_content_type_is (ct, "text", "html")) {
-			if (emp_pp->mode != PREFER_HTML) {
-				if (emp_pp->show_suppressed) {
-					make_part_attachment (
-						parser, sp, part_id, FALSE,
-						cancellable, &work_queue);
-				}
-			} else {
+			if (prefer_html) {
 				e_mail_parser_parse_part (
 					parser, sp, part_id,
 					cancellable, &work_queue);
+			} else if (emp_pp->show_suppressed) {
+				make_part_attachment (
+					parser, sp, part_id, FALSE,
+					cancellable, &work_queue);
 			}
 
+			has_html = TRUE;
+
 		} else if (camel_content_type_is (ct, "text", "plain")) {
 			e_mail_parser_parse_part (
 				parser, sp, part_id,
@@ -249,7 +258,7 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 
 		/* Always show calendar part! */
 		} else if (camel_content_type_is (ct, "text", "calendar") ||
-		    camel_content_type_is (ct, "text", "x-calendar")) {
+			   camel_content_type_is (ct, "text", "x-calendar")) {
 
 			/* Hide everything else, displaying
 			 * native calendar part only. */
@@ -260,11 +269,12 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 
 			has_calendar = TRUE;
 
-		/* Multiparts can represent a text/html with inline images or so */
+		/* Multiparts can represent a text/html message
+		 * with other things like embedded images, etc. */
 		} else if (camel_content_type_is (ct, "multipart", "*")) {
 			GQueue inner_queue = G_QUEUE_INIT;
 			GList *head, *link;
-			gboolean has_html = FALSE;
+			gboolean multipart_has_html = FALSE;
 
 			e_mail_parser_parse_part (
 				parser, sp, part_id, cancellable, &inner_queue);
@@ -276,12 +286,12 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 				EMailPart *mail_part = link->data;
 
 				if (strstr (mail_part->id, ".text_html") != NULL) {
-					has_html = TRUE;
+					multipart_has_html = TRUE;
 					break;
 				}
 			}
 
-			if (has_html && (emp_pp->mode != PREFER_HTML)) {
+			if (multipart_has_html && !prefer_html) {
 				if (emp_pp->show_suppressed) {
 					e_mail_parser_wrap_as_attachment (
 						parser, sp, part_id,
@@ -293,6 +303,8 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 
 			e_queue_transfer (&inner_queue, &work_queue);
 
+			has_html |= multipart_has_html;
+
 		/* Parse everything else as an attachment */
 		} else {
 			GQueue inner_queue = G_QUEUE_INIT;
@@ -308,10 +320,10 @@ empe_prefer_plain_parse (EMailParserExtension *extension,
 	}
 
 	/* Don't hide the plain text if there's nothing else to display */
-	if (has_calendar || (nparts > 1 && emp_pp->mode == PREFER_HTML))
+	if (has_calendar || (has_html && prefer_html))
 		hide_parts (&plain_text_parts);
 
-	if (!g_queue_is_empty (&plain_text_parts) && !g_queue_is_empty (&work_queue) && nparts > 1) {
+	if (!g_queue_is_empty (&plain_text_parts) && !g_queue_is_empty (&work_queue) && has_html) {
 		/* a text/html part is hidden, but not marked as attachment,
 		 * thus do that now, when there exists a text/plain part */
 		GList *qiter;
