@@ -84,9 +84,10 @@ static void
 settings_mail_reader_constructed (GObject *object)
 {
 	/* Bind properties to settings from an idle callback so the
-	 * EMailReader interface has a chance to be initialized first. */
+	 * EMailReader interface has a chance to be initialized first.
+	 * Prioritize ahead of GTK+ redraws. */
 	g_idle_add_full (
-		G_PRIORITY_DEFAULT_IDLE,
+		G_PRIORITY_HIGH_IDLE,
 		(GSourceFunc) settings_mail_reader_idle_cb,
 		g_object_ref (object),
 		(GDestroyNotify) g_object_unref);
