@@ -67,6 +67,7 @@ struct _EShellWindowClass {
 	GtkWindowClass parent_class;
 
 	/* Signals */
+	void		(*close_alert)		(EShellWindow *shell_window);
 	void		(*shell_view_created)	(EShellWindow *shell_window,
 						 struct _EShellView *shell_view);
 
@@ -134,11 +135,6 @@ gboolean	e_shell_window_get_toolbar_visible
 void		e_shell_window_set_toolbar_visible
 						(EShellWindow *shell_window,
 						 gboolean toolbar_visible);
-const gchar *	e_shell_window_get_toolbar_new_prefer_item
-						(EShellWindow *shell_window);
-void		e_shell_window_set_toolbar_new_prefer_item
-						(EShellWindow *shell_window,
-						 const gchar *prefer_item);
 
 /* These should be called from the shell backend's window_created() handler. */
 
